/************************************************************************/
/* requests.h - DNS request management                                   */
/************************************************************************/
/*
 *  Copyright 2021-2023 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _REQUESTS_H_
#define _REQUESTS_H_


#include <stdint.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>

//#include <cutil/slist.h>



//typedef enum {
//	REQUEST_FRONTEND_PROTO_UDP	= 0,
//	REQUEST_FRONTEND_PROTO_TCP,
//	_REQUEST_FRONTEND_PROTO_MAX
//} request_frontend_proto;



typedef struct _request_t request_t;
struct _request_t {
	// DNS request info
	uint16_t	req_id;

	// source identification for UDP responses
	uint32_t	src_ip;
	uint16_t	src_port;
	struct sockaddr	src_sockaddr;
	socklen_t	src_salen;

	// frontend/backend info
	int		frontend_fd;
	int		backend_fd;

	// timestamps
	struct timespec	start_ts;
};


typedef struct _request_table_t request_table_t;
struct _request_table_t {
	uint32_t	width;
//	SList		*list_table;
	request_t	*table;
};



request_table_t *request_table_new(uint32_t width);
int request_table_free(request_table_t *table);
request_t *request_table_get_entry(request_table_t *table, uint16_t id);
void request_table_clear_entry(request_table_t *table, uint16_t id);
void request_reset(request_t *request);
void dump_request(request_t *request);

#endif



